% Clear the workspace, the command window, and close all figures.
clear;
clc;
close all;

% profile on;

% Define the file path for the data file to be loaded.
dir = '../../../Data/DataMerging.mat';

% Load the data file.
load(dir);
%% Compute PDRF values for each event.

% Initialize variables to store the results of each event.
num_events = 18;
PDRF = cell(1, num_events);
PDRF_event = cell(1, num_events);
PR_event = cell(1, num_events);
PR_conti = cell(1, num_events);
PDRF_average = zeros(1, num_events);
PR_event_average = zeros(1, num_events);
ElapsedTime = [];
% Loop over each event.
E = {E1, E2, E3, E4, E5, E6, E7, E8, E9, ...
    E10, E11, E12, E13, E14, E15, E16, E17, E18}; % Store events in a cell array
% tic
% Define model parameters
sig_x = 2.01;
sig_y = 0.02;
    % Prediction Horizon
T1 = 0.5;
T2 = 1;
T3 = 2;
T4 = 3;
T=[T1,T2,T3,T4];
warning('off','all')
warning
for nn = 1:num_events
    
    % Extract the x, y, vx, vy, ax, and ay components of the ego and object vehicles.
    event = E{nn}; % Access the event data

    x_s = event.x_s;
    y_s = event.y_s;
    vx_s = event.vx_s;
    vy_s = event.vy_s;
    ax_s = event.ax_s;
    ay_s = event.ay_s;
    
    x_n = event.x_n;
    y_n = event.y_n;
    vx_n = event.vx_n;
    vy_n = event.vy_n;
    ax_n = event.ax_n;
    ay_n = event.ay_n;
    
    % Extract the PR_event data for the current event type.
    PR_event{nn} = event.PR_event;
    
    % Compute the PDRF values for each sample of the current event using a parallel for loop.
    len = length(x_s);
    PDRF_storage = zeros(1, len);
    tic

    parfor i = 1:len
        warning('off','all')
        [PROB05, ~,~,~,p_ax_temp1,p_ay_temp1] = C_PROB(x_s(i), y_s(i), vx_s(i), vy_s(i), x_n(i), y_n(i), vx_n(i), vy_n(i), ax_n(i), ay_n(i), T1, sig_x, sig_y);
        [PROB1, ~,~,~,p_ax_temp2,p_ay_temp2] = C_PROB(x_s(i), y_s(i), vx_s(i), vy_s(i), x_n(i), y_n(i), vx_n(i), vy_n(i), ax_n(i), ay_n(i), T2, sig_x, sig_y);% the collision probability
        [PROB2, ~,~,~,p_ax_temp3,p_ay_temp3] = C_PROB(x_s(i), y_s(i), vx_s(i), vy_s(i), x_n(i), y_n(i), vx_n(i), vy_n(i), ax_n(i), ay_n(i), T3, sig_x, sig_y);
        [PROB3, ~,~,~,p_ax_temp4,p_ay_temp4]= C_PROB(x_s(i), y_s(i), vx_s(i), vy_s(i), x_n(i), y_n(i), vx_n(i), vy_n(i), ax_n(i), ay_n(i), T4, sig_x, sig_y);
        [PROB(i),idx] = max([PROB05,PROB1,PROB2,PROB3]);
        
        p_ax = [p_ax_temp1, p_ax_temp2, p_ax_temp3, p_ax_temp4];
        p_ay = [p_ay_temp1, p_ay_temp2, p_ay_temp3, p_ay_temp4];
%         PDRF_storage(i)=PROB(i);
        PDRF_storage(i)=0.5*1439.3*(2590/(2590+1439.3)).^2*((vx_s(i)-(vx_n(i)+p_ax(idx).*T(idx))).^2+(vy_s(i)-(vy_n(i)+p_ay(idx).*T(idx))).^2)*PROB(i);
%         PDRF_storage(i)=0.5*1439.3*(2590/(2590+1439.3)).^2*((vx_s(i)-vx_n(i)).^2+(vy_s(i)-vy_n(i)).^2)*PROB(i);

%     Using the collision probability only can obtain a much better
%     performance
    end
    
    ElapsedTime(end+1) = toc/4; 
    % Store the PDRF values and compute the PDRF_event and PR_conti values for the current event.
    PDRF{nn} = PDRF_storage;
    len_risk = length(PR_event{nn});
    PR_conti_temp_storage = event.PR_conti;
    
    % Find the maximum value as the event-based perceived risk estimation
    indices = 1:50:len;
    PDRF_event_storage = arrayfun(@(i) max(PDRF_storage(i:min(i+49, len))), indices);
    PR_conti_peak_storage = arrayfun(@(i) max(PR_conti_temp_storage(i:min(i+49, len))), indices);

    PDRF_event{nn} = PDRF_event_storage;
    PDRF_average(nn) = mean(PDRF_event_storage);
    PR_conti{nn} = PR_conti_peak_storage;
    PR_event_average(nn) = mean(PR_event{nn});
    
    % Print the current event number to the console.
    disp(nn)
end
% toc
% Concatenate the PDRF_event, PR_event, and PR_conti cell arrays into matrices.
PDRF_event_mat = cell2mat(PDRF_event);
PR_event_mat = cell2mat(PR_event);
PR_conti_peak_mat = cell2mat(PR_conti);
%% Plot the data and fit linear regression models.

% Create a new figure and plot the PDRF_event_mat versus PR_event_mat as circles with the MarkerEdgeColor set to [0.85,0.33,0.10].
% Plot the PDRF_average versus PR_event_average as circles with a black face and edge color.
close all;
figure;
plot(PR_event_mat, 10*PDRF_event_mat/max(PDRF_event_mat), 'o', 'MarkerEdgeColor', [0.85, 0.33, 0.10]); hold on
plot(PR_event_average, 10*PDRF_average/max(PDRF_average), 'o', 'MarkerFaceColor', 'k', 'MarkerEdgeColor', 'k');

% Set the x-axis label to "Event-based perceived risk," the y-axis label to "Predicted perceived risk," and the limits for both axes from 0 to 10.
xlabel('Event-based perceived risk');
ylabel('Predicted perceived risk');
ylim([0 10]);
xlim([0 10]);

% Plot a diagonal line from the origin.
plot(0:0.1:10, 0:0.1:10);

% Fit linear regression models to the data.
mdl = fitlm(PR_event_average, PDRF_average);
mdl2 = fitlm(PR_event_mat, PDRF_event_mat);

% Print the R-squared values for both models to the console.
fprintf('Model using PR_event_average and PDRF_average:\n');
fprintf('Rsquared.Ordinary: %.4f\n', mdl.Rsquared.Ordinary);
fprintf('Rsquared.Adjusted: %.4f\n\n', mdl.Rsquared.Adjusted);

fprintf('Model using PR_event_mat and PDRF_event_mat:\n');
fprintf('Rsquared.Ordinary: %.4f\n', mdl2.Rsquared.Ordinary);
fprintf('Rsquared.Adjusted: %.4f\n\n', mdl2.Rsquared.Adjusted);

% Compute the root mean squared error (RMSE) and the mean absolute error (MAE) for both PR_event and PR_conti_peak, and print the results to the console.
rmse_event = sqrt(mean((PR_event_mat - 10*PDRF_event_mat/max(PDRF_event_mat)).^2));
rmse_peak = sqrt(mean((PR_conti_peak_mat - 10*PDRF_event_mat/max(PDRF_event_mat)).^2));
AE_event = mean(abs(PR_event_mat - 10*PDRF_event_mat/max(PDRF_event_mat)));
AE_peak = mean(abs(PR_conti_peak_mat - 10*PDRF_event_mat/max(PDRF_event_mat)));

fprintf('Root mean squared error for PR_event and PDRF_event: %.4f\n', rmse_event);
fprintf('Root mean squared error for PR_conti_peak and PDRF_event: %.4f\n', rmse_peak);
fprintf('Mean absolute error for PR_event and PDRF_event: %.4f\n', AE_event);
fprintf('Mean absolute error for PR_conti_peak and PDRF_event: %.4f\n', AE_peak);

% 
% profile off;
% profile viewer;
% Calculate the time comsumption per step
ComputationCost=mean(ElapsedTime/len);
fprintf('Computation Cost for each step: %.4f\n', ComputationCost);
warning('on','all');