# README 2D computational perceived risk modelling

## Description
This MATLAB implementation consists of one `Data` folder and three code folders `ImpForDatasetMerging`, `ImpForDatasetObsAvoid` and `Validation`. The three code folders contains the code and their implementation of perceived risk model `Potential collision avoidance difficulty model (PCAD)`, `Regression perceived risk model (RPR)`, `Perceived probabilistic driving risk field model (PPDRF)` and `Driving risk field model (DRF)` for `Dataset Merging`, `Dataset Obstacle Avoidance` and `Cross Validation` . Additionally, `RadarChart` contains `RadarChart.m` to generate a radar chart for performance comparison.

## Dependencies
- MATLAB

### PCAD
`PCAD_computation.m`: The m file that combines three sub-functions to compute the model performance of PCAD in the corresponding dataset.

#### Sub-functions
1. `PCAD_function.m`: This is the main function calculating the PCAD value based on the position, velocity, and acceleration values of the subject and neighboring vehicles. It also considers various model parameters such as Gaussian density functions and accumulation time for perceived velocity computation for both vehicles. This function calls `d_dot_cal` and `BearingRate_prod`.

2. `BearingRate_prod.m`: Computes the looming product based on the bearing rate between different points of the subject and neighboring vehicles. It considers the positions and velocities of the vehicles, as well as the angular velocity (Yaw rate/omega).

3. `d_dot_cal.m`: Calculates the dot product of the distance vectors (d_dot) between the subject vehicle and the neighboring vehicle. Takes as input the position and velocity values of both vehicles.

#### Usage
To get PCAD performance in two datasets and for validation, run `PCAD_computation.m` in corresponding folder. To use PCAD code alone, provide the required input parameters for the `PCAD_function` and run it in MATLAB. The function will return the PCAD value, which can be used to decide on the optimal velocity change for the subject vehicle to avoid collisions.

#### Example
```matlab
% Input parameters
x_s = 10;
x_n = 15;
y_s = 5;
y_n = 7;
vx_s = 20;
vx_n = 18;
vy_s = 10;
vy_n = 12;
ax_s = 1;
ax_n = 1;
ay_s = 1;
ay_n = 1;
sigxs = 0.8;
sigys = 1.7;
sigxn = 4.28;
sigyn = 3.86;
tps = 0.13;
tpn = 0.01;
alpha = 0.52;

% Calculate PCAD value
PCAD = PCAD_function(x_s, x_n, y_s, y_n, vx_s, vx_n, vy_s, vy_n, ax_s, ax_n, ay_s, ay_n, sigxs, sigys, sigxn, sigyn, tps, tpn, alpha);

% Output: PCAD value
disp(PCAD);
```
### RPR
`RPR_computation.m`: The m file computes the model performance of RPR in corresponding dataset.

#### Usage
To get RPR performance in two datasets and for validation, run `RPR_computation.m` in corresponding folder. 


### PPDRF
`PPDRF_computation.m`: The m file that combines three sub-functions to compute the model performance of PPDRF in the corresponding dataset.

#### Sub-functions
1. `C_PROB.m`: This is the core function calculating the collision likelihood value based on the position, velocity, and acceleration values of the subject and neighboring vehicles. This function calls `reachable` and `TriIntegral`.

2. `reachable.m`: A function that calculates the reachable space for a vehicle given its current position, velocity, and acceleration constraints.

3. `TriIntegral.m`: A function that calculates the integral of a function over a triangle.

#### Usage
To get PPDRF performance in two datasets and for validation, run `PPDRF_computation.m` in corresponding folder. 

####
The `TriIntegral.m` function is adapted from the MATLAB official forum. The orignal source can be found [here](https://nl.mathworks.com/matlabcentral/answers/430420-integrate-a-function-over-a-triangle-area#answer_347505).

### DRF
`DRF_computation.m`: The m file computes the model performance of DRF in the corresponding dataset.

#### Sub-function
1. `DRF_cal.m`: This is the core function calculating the perceived driving risk based on the position, and velocity values of the subject and neighboring vehicles. This function calls `TriIntegral`.

2. `TriIntegral.m`: A function that calculates the integral of a function over a triangle.

#### Usage
To get DRF performance in two datasets and for validation, run `DRF_computation.m` in corresponding folder. 


