% Clear the workspace, the command window, and close all figures.
clear;
clc;
close all;

% profile on;

% Define the file path for the data file to be loaded.
dir = '../../../Data/DataMerging.mat';

% Load the data file.
load(dir);
%% RPR computation
% Initialize variables to store the results of each event.
num_events = 18;
RPR = cell(1, num_events);
RPR_event = cell(1, num_events);
PR_event = cell(1, num_events);
PR_conti = cell(1, num_events);
RPR_average = zeros(1, num_events);
PR_event_average = zeros(1, num_events);
ElapsedTime = [];
% Loop over each event.
E = {E1, E2, E3, E4, E5, E6, E7, E8, E9, ...
    E10, E11, E12, E13, E14, E15, E16, E17, E18}; % Store events in a cell array
% Define model parameters
x=[20.7, -3.68, 0];
for nn=1:18 % 9x2 = 18 events  
    event = E{nn}; % Access the event data

    x_s = event.x_s;
    y_s = event.y_s;
    vx_s = event.vx_s;
    vy_s = event.vy_s;
    ax_s = event.ax_s;
    ay_s = event.ay_s;
    
    x_n = event.x_n;
    y_n = event.y_n;
    vx_n = event.vx_n;
    vy_n = event.vy_n;
    ax_n = event.ax_n;
    ay_n = event.ay_n;
    
    gap=x_n-x_s;
    gap(gap<=0.1)=0.1;
    ax_n(ax_n>0)=0;
    % Extract the PR_event data for the current event type.
    PR_event{nn} = event.PR_event;
    
    % Compute the RPR values for each sample of the current event using a parallel for loop.
    len = length(x_s);
    tic
    RPR_storage=(x(1)+x(2).*log(gap)+x(3).*ax_n);
    ElapsedTime(end+1) = toc; 
    RPR_storage(RPR_storage==0)=NaN;
    % Store the RPR values and compute the RPR_event and PR_conti values for the current event.
    RPR{nn} = RPR_storage;
    len_risk = length(PR_event{nn});
    PR_conti_temp_storage = event.PR_conti;
    
    % Find the maximum value as the event-based perceived risk estimation
    indices = 1:50:len;
    RPR_event_storage = arrayfun(@(i) max(RPR_storage(i:min(i+49, len))), indices);
    PR_conti_peak_storage = arrayfun(@(i) max(PR_conti_temp_storage(i:min(i+49, len))), indices);


    RPR_event{nn} = RPR_event_storage;
    RPR_average(nn) = mean(RPR_event_storage);
    PR_conti{nn} = PR_conti_peak_storage;
    PR_event_average(nn) = mean(PR_event{nn});
    
    % Print the current event number to the console.
    disp(nn)
end
RPR_event_mat=cell2mat(RPR_event);
PR_event_mat=cell2mat(PR_event);
PR_conti_peak_mat=cell2mat(PR_conti);
%% Plot the data and fit linear regression models.

% Create a new figure and plot the RPR_event_mat versus PR_event_mat as circles with the MarkerEdgeColor set to [0.85,0.33,0.10].
% Plot the RPR_average versus PR_event_average as circles with a black face and edge color.
figure;
plot(PR_event_mat, 3+10*(RPR_event_mat-min(RPR_event_mat))/(max(RPR_event_mat)-min(RPR_event_mat)), 'o', 'MarkerEdgeColor', [0.85, 0.33, 0.10]); hold on
plot(PR_event_average, 3+10*(RPR_average-min(RPR_event_mat))/(max(RPR_event_mat)-min(RPR_event_mat)), 'o', 'MarkerFaceColor', 'k', 'MarkerEdgeColor', 'k');

% Set the x-axis label to "Event-based perceived risk," the y-axis label to "Predicted perceived risk," and the limits for both axes from 0 to 10.
xlabel('Event-based perceived risk');
ylabel('Predicted perceived risk');
ylim([0 10]);
xlim([0 10]);

% Plot a diagonal line from the origin.
plot(0:0.1:10, 0:0.1:10);

% Fit linear regression models to the data.
mdl = fitlm(PR_event_average, RPR_average);
mdl2 = fitlm(PR_event_mat, RPR_event_mat);

% Print the R-squared values for both models to the console.
fprintf('Model using PR_event_average and RPR_average:\n');
fprintf('Rsquared.Ordinary: %.4f\n', mdl.Rsquared.Ordinary);
fprintf('Rsquared.Adjusted: %.4f\n\n', mdl.Rsquared.Adjusted);

fprintf('Model using PR_event_mat and RPR_event_mat:\n');
fprintf('Rsquared.Ordinary: %.4f\n', mdl2.Rsquared.Ordinary);
fprintf('Rsquared.Adjusted: %.4f\n\n', mdl2.Rsquared.Adjusted);

% Compute the root mean squared error (RMSE) and the mean absolute error (MAE) for both PR_event and PR_conti_peak, and print the results to the console.
rmse_event = sqrt(mean((PR_event_mat - RPR_event_mat).^2));
rmse_peak = sqrt(mean((PR_conti_peak_mat - RPR_event_mat).^2));
AE_event = mean(abs(PR_event_mat - RPR_event_mat));
AE_peak = mean(abs(PR_conti_peak_mat - RPR_event_mat));

fprintf('Root mean squared error for PR_event and RPR_event: %.4f\n', rmse_event);
fprintf('Root mean squared error for PR_conti_peak and RPR_event: %.4f\n', rmse_peak);
fprintf('Mean absolute error for PR_event and RPR_event: %.4f\n', AE_event);
fprintf('Mean absolute error for PR_conti_peak and RPR_event: %.4f\n', AE_peak);

% 
% profile off;
% profile viewer;
% Calculate the time comsumption per step
ComputationCost=mean(ElapsedTime/len*1000);
fprintf('Computation Cost for each step: %.4f\n', ComputationCost);
